<?php
namespace Ep\Woo\Form;

use Ep\Woo\Product\Grouped;
use Ep\Woo\Form\Element\AttributeList;
use Ep\Woo\Product\ProductInterface;
use Ep\Woo\Product\Variable;
use Laminas\Form\FieldsetInterface;
use Laminas\Form\Form;

/**
 * Obrazac za dodavanje proizvoda u košaricu.
 *
 * @package Ep\Woo\Form
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class AddToCart extends Form
{
    /**
     * Inicijalizacija.
     */
    public function init()
    {
        parent::init();

        $this->setAttribute('id', 'atc_' . substr(md5(rand() . time() * 24), 0, 8));

        $this->add([
            'type' => 'submit',
            'name' => 'add-to-cart',
            'options' => [
                'label' => __('Add to cart', 'woocommerce'),
            ],
            'attributes' => [
                'class' => 'add-to-cart__button btn-alt',
            ],
        ]);
    }

    /**
     * @param \WC_Product|ProductInterface $object
     *
     * @return \Laminas\Form\Fieldset|\Laminas\Form\FieldsetInterface
     */
    public function setObject($object)
    {
        if (!$object instanceof ProductInterface) {
            throw new \RuntimeException(sprintf(
                "Objekt mora implementirati ProductInterface, a dobio sam '%s'.",
                is_object($object) ? get_class($object) : gettype($object)
            ));
        }

        $this->setAttribute('data-product-id', $object->getId());
        $this->setAttribute('data-product-type', $object->getType());

        if ($this->has('quantity')) {
            $this->remove('quantity');
        }

        if ($object instanceof Variable) {
            $this->setVariableProduct($object);
        }

        if ($object instanceof Grouped) {
            $this->setGroupedProduct($object);
        } else if ($object->isPurchasable()) {
            $this->add([
                'type' => 'woo.element.quantity',
                'name' => 'quantity',
                'options' => [
                    'label' => __('Quantity', 'woocommerce'),
                ],
                'attributes' => [
                    'value' => $object->getMinPurchaseQuantity(),
                    'min' => $object->getMinPurchaseQuantity(),
                    'max' => $object->getMaxPurchaseQuantity(),
                    'step' => $object->getPurchaseStep(),
                ],
            ]);
        }

        $this->add([
            'type' => 'woo.element.productenquiry',
            'name' => 'enquiry',
            'attributes' => [
                'class' => 'btn btn-alt product-enquiry add-to-cart__button',
                'data-product-name' => $object->getName(),
                'data-product-sku' => $object->getSku(),
            ],
        ]);

        return parent::setObject($object);
    }

    /**
     * @param Grouped $product
     */
    protected function setGroupedProduct(Grouped $product)
    {
        if ($this->has('children')) {
            $this->remove('children');
        }

        $this->add([
            'type' => 'fieldset',
            'name' => 'children',
        ]);

        /** @var FieldsetInterface $children */
        $children = $this->get('children');

        foreach ($product->getChildren() as $child) {
            $old = $GLOBALS['product'];
            $GLOBALS['product'] = $child;

            ob_start();
            woocommerce_template_loop_price();
            $price = ob_get_clean();

            $children->add([
                'type' => 'woo.element.quantity',
                'name' => sprintf('quantity[%d]', $child->getId()),
                'options' => [
                    'product_id' => $child->getId(),
                    'product_name' => $child->getName(),
                    'product_price' => $price,
                    'real_product_price' => $child->get_price(),
                    'product_stock' => wc_get_stock_html($child),
                    'product_sku' => $child->getSku(),
                    'product_url' => get_permalink($child->getId()),
                    'product_visible' => $child->isVisible(),
                ],
                'attributes' => [
                    'value' => 0,
                    'min' => 0,
                    'max' => $child->getMaxPurchaseQuantity(),
                    'step' => $child->getMinPurchaseQuantity(),

                    'data-product-id' => $child->getId(),
                ],
            ]);

            $GLOBALS['product'] = $old;
        }
    }

    /**
     * @param Variable $product
     *
     * @return void|\Laminas\Form\Element|\Laminas\Form\ElementInterface
     */
    protected function setVariableProduct(Variable $product)
    {
        if ($this->has('attributes')) {
            $this->remove('attributes');
        }

        $this->add([
            'type' => 'woo.element.attributelist',
            'name' => 'attributes',
        ]);

        /** @var AttributeList $attributes */
        $attributes = $this->get('attributes');
        $attributes->setProduct($product);

        $variations = [];
        $available_variations = $product->get_available_variations();

        foreach ($available_variations as $variation) {
            $key = implode('|', $variation['attributes']);
            $variations[$key] = $variation;
        }

        $this->setAttribute('data-product-variations', $variations);
    }
}
