<?php
namespace Ep\Woo\Plugin\Coupon;

use Ak\Wp\Plugin\PluginInterface;
use WC_Coupon;

/**
 * Kuponi za korisnike registrirane prije.
 *
 * Kuponima dodajemo novo ograničenje korištenja. Moguće je postaviti datum prije kojeg
 * korisnik mora biti registriran da bi mogao iskoristiti kupon.
 *
 * @package Ep\Woo\Plugin
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class UserRegisteredBefore implements PluginInterface
{
    const META_KEY = '_user_registered_before';

    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function attach(): void
    {
        add_filter('woocommerce_coupon_is_valid', [$this, 'woo_coupon_is_valid']);

        if (defined('WP_ADMIN')) {
            add_action('save_post_shop_coupon', [$this, 'save_post_shop_coupon']);
            add_action('woocommerce_coupon_options_usage_restriction', [$this, 'woo_coupon_options_usage_restriction']);
        }
    }

    /**
     * Poziva se nakon što je kupon spremljen.
     *
     * @param int $post_ID
     */
    public function save_post_shop_coupon(int $post_ID): void
    {
        if (isset($_POST[self::META_KEY])) {
            update_post_meta($post_ID, self::META_KEY, $_POST[self::META_KEY]);
        }
    }

    /**
     * Da li je dati kupon validan?
     *
     * @param bool $valid
     * @param WC_Coupon $coupon
     *
     * @return bool
     */
    public function woo_coupon_is_valid(bool $valid, WC_Coupon $coupon): bool
    {
        $before = get_post_meta($coupon->get_id(), self::META_KEY, true);

        if (!empty($before)) {
            $user = wp_get_current_user();

            if (0 === $user->ID) {
                return false;
            }

            return strtotime($before) >= strtotime($user->user_registered);
        }

        return $valid;
    }

    /**
     * Renderira polje za unost datuma u adminu.
     *
     * @param int $coupon_ID
     */
    public function woo_coupon_options_usage_restriction(int $coupon_ID): void
    {
        $before = get_post_meta($coupon_ID, self::META_KEY, true);

        woocommerce_wp_text_input([
            'id'                => 'users_registered_before',
            'value'             => esc_attr($before),
            'label'             => 'Dozvoli korisnicima registriranim prije',
            'placeholder'       => 'YYYY-MM-DD',
            'description'       => '',
            'class'             => 'date-picker',
            'custom_attributes' => [
                'pattern' => apply_filters(
                    'woocommerce_date_input_html_pattern',
                    '[0-9]{4}-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[01])'
                ),
            ],
        ]);
    }
}