<?php
namespace Ep\Woo\Product\View\Helper;

use DateTime;
use Ep\Woo\Product\ProductInterface;
use Ep\Woo\Product\Simple;
use Exception;
use Laminas\View\Helper\AbstractHelper;
use RuntimeException;
use WP_Term;

/**
 * Renderira SaleBadge element.
 *
 * @package Ep\Woo\Produc\View\Helper
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class SaleBadge extends AbstractHelper
{
    protected $saleConfig;

    public function __construct(array $saleConfig)
    {
        $this->saleConfig = $saleConfig;
    }

    /**
     * Renderira element.
     *
     * @param $productOrCategory
     *
     * @return string
     * @throws Exception
     */
    public function __invoke($productOrCategory): string
    {
        if ($productOrCategory instanceof ProductInterface) {
            return $this->renderProduct($productOrCategory);
        } else if ($productOrCategory instanceof WP_Term) {
            return $this->renderCategory($productOrCategory);
        }

        throw new RuntimeException(sprintf("SaleBadge helper ne zna rendati '%s'.", get_class($productOrCategory)));
    }

    /**
     * @param WP_Term $category
     * @return string
     * @throws Exception
     */
    public function renderCategory(WP_Term $category): string
    {
        $text = get_term_meta($category->term_id, '_sale_badge_text', true) ?: '';
        $size = strtolower(get_term_meta($category->term_id, '_sale_badge_size', true) ?: 'none');

        if (empty($text) || 'none' === $size || $this->saleEnded()) {
            return '';
        }

        add_action('wp_footer', [$this, 'renderSvgPath']);
        return sprintf('<span class="sale-badge-category sale-badge-category--%s">%s</span>', $size, $text);
    }

    /**
     * Renderira sale badge za proizvod.
     *
     * @param ProductInterface|Simple $product
     * @return string
     */
    public function renderProduct(ProductInterface $product): string
    {
        if (!$product->isOnSale() || $product->is_price_hidden() || !$product->getRegularPrice()) {
            return '';
        }

        $percentage = 100 - round($product->getSalePrice() / $product->getRegularPrice() * 100);

        add_action('wp_footer', [$this, 'renderSvgPath']);
        return sprintf('<span class="sale-badge">-%d%%</span>', $percentage);
    }

    /**
     * @return bool
     * @throws Exception
     */
    protected function saleEnded(): bool
    {
        $endSale = $this->saleConfig['date_end_sale'];

        $saleIsActive = (bool)$this->saleConfig['active_sale'];

        if ($saleIsActive) {
            $now = new DateTime('now');
            $end = new DateTime($endSale);

            return ( $now > $end );
        }

        return true;
    }

    public function renderSvgPath()
    {
        ?>
        <div style="position: absolute; top: -1000px;">
            <svg>
                <defs>
                    <!--suppress HtmlUnknownTag -->
                    <clipPath id="badgepath" clipPathUnits="objectBoundingBox">
                        <!--suppress CheckEmptyScriptTag -->
                        <path class="cls-1" d="
                    M 0.545454545,0.992727273
                    l 0.404090909 -0.161590909
                    C 0.981590909,0.818181818,1,0.801136364,1,0.783181818
                    v -0.75
                    C 1,0.015,0.959545455,0,0.911136364,0
                    H 0.088863636
                    C 0.039318182,0,0,0.015,0,0.032954545
                    v 0.75
                    c 0,0.017954545,0.018409091,0.035681818,0.050909091,0.047954545
                    L 0.454545455,0.992727273
                    A 0.141818182,0.141818182,0,0,0,0.545454545,0.992727273
                    Z
                "/>
                    </clipPath>
                </defs>
            </svg>
        </div>
        <?php
    }
}
